﻿Option Strict Off
Option Explicit On
Module ApiPrtCtrl

    '*************************************************************
    ' WIN32関連
    '*************************************************************

    ' ============================================================
    ' レジストリ操作のための定義
    ' ============================================================

    ' HEKY 定数
    Public Const HKEY_CLASSES_ROOT As Integer = &H80000000
    Public Const HKEY_CURRENT_USER As Integer = &H80000001
    Public Const HKEY_LOCAL_MACHINE As Integer = &H80000002
    Public Const HKEY_USERS As Integer = &H80000003

    ' データタイプ
    Public Enum RegType
        REG_SZ = 1
        REG_DWORD = 4
    End Enum

    ' SECURITY_ATTRIBUTES 構造体

    Public Structure SECURITY_ATTRIBUTES
        ' 構造体のサイズ
        Dim nLength As Integer
        ' オブジェクトに対するセキュリティ記述子を指すポインタ
        Dim lpSecurityDescriptor As Integer
        ' 新しいプロセスを作成したときに返されるハンドルが継承可能かどうかを指定します
        Dim bInheritHandle As Integer
    End Structure

    ' ============================================================
    ' レジストリアクセスのための情報
    ' ============================================================

    ' このキーは不揮発性です
    Private Const REG_OPTION_NON_VOLATILE As Integer = &H0S

    ' サブキーデータの問い合わせを許可します
    Private Const KEY_QUERY_VALUE As Integer = &H1S

    ' サブキーの列挙を許可します
    Private Const KEY_ENUMERATE_SUB_KEYS As Integer = &H8S

    ' 変更の通知を許可します。
    Private Const KEY_NOTIFY As Integer = &H10S

    ' サブキーの作成を許可します
    Private Const KEY_CREATE_SUB_KEY As Integer = &H4S

    ' シンボリックリンクの作成を許可します
    Private Const KEY_CREATE_LINK As Integer = &H20S

    'サブキーデータの設定を許可します
    Private Const KEY_SET_VALUE As Integer = &H2S

    ' KEY_QUERY_VALUE、KEY_ENUMERATE_SUB_KEYS、KEY_NOTIFY、
    ' KEY_CREATE_SUB_KEY、KEY_CREATE_LINK、KEY_SET_VALUE の組み合わせです
    Private Const KEY_ALL_ACCESS As Integer = KEY_QUERY_VALUE Or KEY_ENUMERATE_SUB_KEYS Or KEY_NOTIFY Or KEY_CREATE_SUB_KEY Or KEY_CREATE_LINK Or KEY_SET_VALUE

    ' KEY_QUERY_VALUE、KEY_ENUMERATE_SUB_KEYS、and KEY_NOTIFY の組み合わせです
    Private Const KEY_READ As Integer = KEY_QUERY_VALUE Or KEY_ENUMERATE_SUB_KEYS Or KEY_NOTIFY

    ' KEY_SET_VALUE と KEY_CREATE_SUB_KEY の組み合わせです
    Private Const KEY_WRITE As Integer = KEY_SET_VALUE Or KEY_CREATE_SUB_KEY

    Public Const ERROR_SUCCESS As Short = 0

    ' ============================================================
    ' Registry API prototypes
    ' ============================================================

    Declare Function RegOpenKey Lib "advapi32.dll" Alias "RegOpenKeyA" (ByVal hKey As Integer, ByVal lpSubKey As String, ByRef phkResult As Integer) As Integer

    Declare Function RegOpenKeyEx Lib "advapi32.dll" Alias "RegOpenKeyExA" (ByVal hKey As Integer, ByVal lpSubKey As String, ByVal ulOptions As Integer, ByVal samDesired As Integer, ByRef phkResult As Integer) As Integer

    Declare Function RegQueryValue Lib "advapi32.dll" Alias "RegQueryValueA" (ByVal hKey As Integer, ByVal lpSubKey As String, ByVal lpValue As String, ByRef lpcbValue As Integer) As Integer

    Declare Function RegQueryValueEx Lib "advapi32.dll" Alias "RegQueryValueExA" (ByVal hKey As Integer, ByVal lpValueName As String, ByVal lpReserved As Integer, ByRef lpType As Integer, ByRef lpData As Integer, ByRef lpcbData As Integer) As Integer

    Public Declare Function RegSetValueEx Lib "advapi32.dll" Alias "RegSetValueExA" (ByVal hKey As Integer, ByVal lpValueName As String, ByVal Reserved As Integer, ByVal dwType As Integer, ByVal lpData As String, ByVal cbData As Integer) As Integer

    Declare Function RegCreateKeyEx Lib "advapi32.dll" Alias "RegCreateKeyExA" (ByVal hKey As Integer, ByVal lpSubKey As String, ByVal Reserved As Integer, ByVal lpClass As String, ByVal dwOptions As Integer, ByVal samDesired As Integer, ByRef lpSecurityAttributes As SECURITY_ATTRIBUTES, ByRef phkResult As Integer, ByRef lpdwDisposition As Integer) As Integer

    Declare Function RegCloseKey Lib "advapi32.dll" (ByVal hKey As Integer) As Integer

    'レジストリの値を取得する
    Public Declare Function RegQueryValueExREG_SZ Lib "ADVAPI32" Alias "RegQueryValueExA" (ByVal hKey As Integer, ByVal lpValueName As String, ByVal lpReserved As Integer, ByVal lpType As Integer, ByVal lpData As String, ByRef lpcbData As Integer) As Integer

    Public Declare Function RegSetValueExREG_DWORD Lib "advapi32.dll" Alias "RegSetValueExA" (ByVal hKey As Integer, ByVal lpValueName As String, ByVal Reserved As Integer, ByVal dwType As Integer, ByRef lpData As Integer, ByVal cbData As Integer) As Integer

    '*************************************************************
    ' 定数定義
    '*************************************************************

    ' STARFAX レジストリサブキー
    Public Const SFCS_LMKEY_SFCS As String = "Software\MEGASOFT\SfCs"

    '*************************************************************
    ' STARFAX レジストリパス文字列取得 (最後に'\'無し)
    '*************************************************************
    Function SFCSENV_GetSvcRegPathString(ByRef strKey As String, ByRef strDefault As String) As String

        Dim lRet As Integer

        Dim strRootkey As String
        Dim strSubkey As String
        Dim lHandle As Integer
        Dim lLeng As Integer
        Dim strGet As String

        strGet = strDefault

        strRootkey = CStr(HKEY_LOCAL_MACHINE)
        strSubkey = SFCS_LMKEY_SFCS

        lRet = RegOpenKeyEx(CInt(strRootkey), strSubkey, 0, KEY_READ, lHandle)
        If lRet = ERROR_SUCCESS Then
            strGet = New String(Chr(0), 250)

            lLeng = Len(strGet)

            Call RegQueryValueExREG_SZ(lHandle, strKey, 0, 0, strGet, lLeng)
            strGet = Left(strGet, lLeng - 1)
            Call RegCloseKey(lHandle)
        End If

        lLeng = Len(strGet)

        If strGet <> "" Then
            ' 最後の'\' あれば 除去
            If Right(strGet, 1) = "\" Then
                strGet = Left(strGet, lLeng - 1)
            End If
        End If

        SFCSENV_GetSvcRegPathString = strGet

    End Function

    '*************************************************************
    ' STARFAX レジストリパス文字列設定
    '*************************************************************
    Function SFCSENV_WriteSvcRegPathString(ByRef strKey As String, ByRef strSet As String) As Boolean
        SFCSENV_WriteSvcRegPathString = SFCSENV_WriteSvcRegString(strKey, strSet)
    End Function

    '*************************************************************
    ' STARFAX レジストリ文字列取得
    '*************************************************************
    Function SFCSENV_GetSvcRegString(ByRef strKey As String, ByRef strDefault As String) As String

        Dim lRet As Integer

        Dim strRootkey As String
        Dim strSubkey As String
        Dim lHandle As Integer
        Dim lLeng As Integer
        Dim strGet As String

        strGet = strDefault

        strRootkey = CStr(HKEY_LOCAL_MACHINE)
        strSubkey = SFCS_LMKEY_SFCS

        lRet = RegOpenKeyEx(CInt(strRootkey), strSubkey, 0, KEY_READ, lHandle)
        If lRet = ERROR_SUCCESS Then
            strGet = New String(Chr(0), 250)

            lLeng = Len(strGet)

            Call RegQueryValueExREG_SZ(lHandle, strKey, 0, 0, strGet, lLeng)
            strGet = Left(strGet, lLeng - 1)
            Call RegCloseKey(lHandle)
        End If

        SFCSENV_GetSvcRegString = strGet

    End Function

    '*************************************************************
    ' STARFAX レジストリ文字列設定
    '*************************************************************
    Function SFCSENV_WriteSvcRegString(ByRef strKey As String, ByRef strSet As String) As Boolean

        Dim lRet As Integer

        Dim strRootkey As String
        Dim strSubkey As String
        Dim lHandle As Integer
        Dim lLeng As Integer
        Dim SA As SECURITY_ATTRIBUTES
        Dim lDisposition As Integer

        strRootkey = CStr(HKEY_LOCAL_MACHINE)
        strSubkey = SFCS_LMKEY_SFCS

        lRet = RegCreateKeyEx(CInt(strRootkey), strSubkey, 0, CStr(0), REG_OPTION_NON_VOLATILE, KEY_WRITE, SA, lHandle, lDisposition)
        If lRet = ERROR_SUCCESS Then

            lLeng = Len(strSet) + 1

            Call RegSetValueEx(lHandle, strKey, 0, RegType.REG_SZ, strSet, lLeng)
            Call RegCloseKey(lHandle)
            SFCSENV_WriteSvcRegString = True
        Else
            SFCSENV_WriteSvcRegString = False
        End If

    End Function

    '*************************************************************
    ' STARFAX レジストリ数値取得
    '*************************************************************
    Function SFCSENV_GetSvcRegInt(ByRef strKey As String, ByRef lDefault As Integer) As Integer

        Dim lRet As Integer

        Dim strRootkey As String
        Dim strSubkey As String
        Dim lHandle As Integer
        Dim lLeng As Integer
        Dim lGet As Integer

        lGet = lDefault

        strRootkey = CStr(HKEY_LOCAL_MACHINE)
        strSubkey = SFCS_LMKEY_SFCS

        lRet = RegOpenKeyEx(CInt(strRootkey), strSubkey, 0, KEY_READ, lHandle)
        If lRet = ERROR_SUCCESS Then
            lLeng = 4
            Call RegQueryValueEx(lHandle, strKey, 0, 0, lGet, lLeng)
            Call RegCloseKey(lHandle)
        End If

        SFCSENV_GetSvcRegInt = lGet

    End Function

    '*************************************************************
    ' STARFAX レジストリ文字列設定
    '*************************************************************
    Function SFCSENV_WriteRegInt(ByRef strKey As String, ByRef lnSet As Integer) As Boolean

        Dim lRet As Integer

        Dim strRootkey As String
        Dim strSubkey As String
        Dim lHandle As Integer
        Dim lLeng As Integer
        Dim SA As SECURITY_ATTRIBUTES
        Dim lDisposition As Integer

        strRootkey = CStr(HKEY_LOCAL_MACHINE)
        strSubkey = SFCS_LMKEY_SFCS

        lRet = RegCreateKeyEx(CInt(strRootkey), strSubkey, 0, CStr(0), REG_OPTION_NON_VOLATILE, KEY_WRITE, SA, lHandle, lDisposition)
        If lRet = ERROR_SUCCESS Then
            lLeng = 4
            Call RegSetValueExREG_DWORD(lHandle, strKey, 0, RegType.REG_DWORD, lnSet, lLeng)
            Call RegCloseKey(lHandle)
            SFCSENV_WriteRegInt = True
        Else
            SFCSENV_WriteRegInt = False
        End If

    End Function

    '*************************************************************
    ' STARFAX 印刷: ファイル出力フォルダ内のクリア
    '*************************************************************
    Function SFCSPRN_ClearOutFolder(ByRef strOutFolder As String, ByRef strDocName As String) As Boolean

        Dim bRet As Boolean
        Dim strWild As String
        Dim strGetFile As String

        bRet = True
        strWild = strOutFolder & "\" & strDocName & "*.*"

        Do Until Dir(strWild) = ""
            strGetFile = strOutFolder & "\" & Dir(strWild)
            If SUB_CheckReadFile(strGetFile) = False Then
                bRet = False
                GoTo STEP_CANSEL
            End If
            Kill(strGetFile)
            bRet = True
        Loop

STEP_CANSEL:

        SFCSPRN_ClearOutFolder = bRet

    End Function

    '*************************************************************
    ' STARFAX 印刷: ファイル出力フォルダ内の印刷完了ファイルの取得
    '*************************************************************
    Function SFCSPRN_GetPrintFileName(ByRef strOutFolder As String, ByRef strDocName As String) As String

        Dim strRet As String
        Dim strWild As String
        Dim strGetFile As String

        strRet = ""
        strWild = strOutFolder & "\" & strDocName & "*.*"

        Do Until Dir(strWild) = ""
            strGetFile = strOutFolder & "\" & Dir(strWild)
            If SUB_CheckReadFile(strGetFile) = True Then
                strRet = strGetFile
                GoTo STEP_CANSEL
            End If
        Loop

STEP_CANSEL:

        SFCSPRN_GetPrintFileName = strRet

    End Function

    '*************************************************************
    ' ファイル読み込み可能確認
    '*************************************************************
    Function SUB_CheckReadFile(ByRef strFileName As String) As Boolean

        Select Case Err.Number
            Case Is < 0
                Error (5)
            Case 1
                GoTo STEP_Error
        End Select
        FileOpen(1, strFileName, OpenMode.Binary, OpenAccess.Read)
        FileClose(1)
        SUB_CheckReadFile = True
        GoTo STEP_Exit

STEP_Error:

        SUB_CheckReadFile = False

STEP_Exit:

    End Function
End Module